/*
 * Copyright (C) 2013 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
//
package seniordesign.harambe;

import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Binder;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.List;
import java.util.UUID;

import seniordesign.harambe.ui.preferenceValues;

import static android.content.ContentValues.TAG;

/**
 * Service for managing connection and data communication with a GATT server hosted on a
 * given Bluetooth LE device.
 */
public class BluetoothLeService extends Service {
    private final static String TAG2 = "life";
    private final static String tag = BluetoothLeService.class.getSimpleName();
    boolean running = false;
    private BluetoothManager mBluetoothManager;
    private BluetoothAdapter mBluetoothAdapter;
    private String mBluetoothDeviceAddress;
    private BluetoothGatt mBluetoothGatt;
    SharedPreferences prefs;
    private int mConnectionState = STATE_DISCONNECTED;
    private BluetoothGattCharacteristic mNotifyCharacteristic;
    boolean check = true;
    private static final int STATE_DISCONNECTED = 0;
    private static final int STATE_CONNECTING = 1;
    private static final int STATE_CONNECTED = 2;

    public final static String ACTION_GATT_CONNECTED =
            "seniordesign.harambe.ACTION_GATT_CONNECTED";
    public final static String ACTION_GATT_DISCONNECTED =
            "seniordesign.harambe.ACTION_GATT_DISCONNECTED";
    public final static String ACTION_GATT_SERVICES_DISCOVERED =
            "seniordesign.harambe.ACTION_GATT_SERVICES_DISCOVERED";
    public final static String ACTION_DATA_AVAILABLE =
            "seniordesign.harambe.ACTION_DATA_AVAILABLE";
    public final static String EXTRA_DATA =
            "seniordesign.harambe.EXTRA_DATA";

    public final static UUID UUID_PROJECT_ZERO =
            UUID.fromString("F0001110-0451-4000-B000-000000000000");
    public final static UUID DATA_SERVICE = UUID.fromString("F0001130-0451-4000-B000-000000000000");
    //  UUID.fromString(SampleGattAttributes.HEART_RATE_MEASUREMENT);
    //"F0001111-0451-4000-B000-000000000000";

    // Implements callback methods for GATT events that the app cares about.  For example,
    // connection change and services discovered.
    private final BluetoothGattCallback mGattCallback = new BluetoothGattCallback() {
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            String intentAction;
            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
            SharedPreferences.Editor editor = prefs.edit();
            if (prefs.getBoolean(preferenceValues.CHECKDATA.name(), false)){
                readData();
            }
            if (newState == BluetoothProfile.STATE_CONNECTED) {
                intentAction = ACTION_GATT_CONNECTED;
                mConnectionState = STATE_CONNECTED;
                broadcastUpdate(intentAction);
                Log.i(tag, "Connected to GATT server.");
                // Attempts to discover services after successful connection.
                Log.i(tag, "Attempting to start service discovery:" +
                        mBluetoothGatt.discoverServices());

            } else if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                intentAction = ACTION_GATT_DISCONNECTED;
                mConnectionState = STATE_DISCONNECTED;
                Log.i(tag, "Disconnected from GATT server.");
                broadcastUpdate(intentAction);
            }
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                broadcastUpdate(ACTION_GATT_SERVICES_DISCOVERED);
            } else {
                Log.w(tag, "onServicesDiscovered received: " + status);
            }
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt,
                                         BluetoothGattCharacteristic characteristic,
                                         int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                broadcastUpdate(ACTION_DATA_AVAILABLE, characteristic);
                Log.w(tag, "money " + characteristic);
            }
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt,
                                            BluetoothGattCharacteristic characteristic) {
            broadcastUpdate(ACTION_DATA_AVAILABLE, characteristic);
        }
    };

    private void broadcastUpdate(final String action) {
        final Intent intent = new Intent(action);
        sendBroadcast(intent);
    }

    private void broadcastUpdate(final String action,
                                 final BluetoothGattCharacteristic characteristic) {
        final Intent intent = new Intent(action);

        // This is special handling for the Heart Rate Measurement profile.  Data parsing is
        // carried out as per profile specifications:
        // http://developer.bluetooth.org/gatt/characteristics/Pages/CharacteristicViewer.aspx?u=org.bluetooth.characteristic.heart_rate_measurement.xml
        if (UUID_PROJECT_ZERO.equals(characteristic.getUuid())) {
            int flag = characteristic.getProperties();

            Log.d(tag, String.format("Received heart rate: %d", 1));
            //     intent.putExtra(EXTRA_DATA, String.valueOf(heartRate));
        } else {
            // For all other profiles, writes the data formatted in HEX.
            final byte[] data = characteristic.getValue();
            if (data != null && data.length > 0) {
                //  Log.i("datadata  ", data[0] + "");
                final StringBuilder stringBuilder = new StringBuilder(data.length);
                for (byte byteChar : data)
                    stringBuilder.append(String.format("%02X ", byteChar));
                intent.putExtra(EXTRA_DATA, new String(data) + "\n" + stringBuilder.toString());

            }
        }
        sendBroadcast(intent);
    }

    public class LocalBinder extends Binder {
        BluetoothLeService getService() {
            return BluetoothLeService.this;
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        return mBinder;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        // After using a given device, you should make sure that BluetoothGatt.close() is called
        // such that resources are cleaned up properly.  In this particular example, close() is
        // invoked when the UI is disconnected from the Service.
        close();
        return super.onUnbind(intent);
    }

    private final IBinder mBinder = new LocalBinder();

    /**
     * Initializes a reference to the local Bluetooth adapter.
     *
     * @return Return true if the initialization is successful.
     */
    public boolean initialize() {
        // For API level 18 and above, get a reference to BluetoothAdapter through
        // BluetoothManager.
        if (mBluetoothManager == null) {
            mBluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
            if (mBluetoothManager == null) {
                Log.e(tag, "Unable to initialize BluetoothManager.");
                return false;
            }
        }

        mBluetoothAdapter = mBluetoothManager.getAdapter();
        if (mBluetoothAdapter == null) {
            Log.e(tag, "Unable to obtain a BluetoothAdapter.");
            return false;
        }

        return true;
    }

    /**
     * Connects to the GATT server hosted on the Bluetooth LE device.
     *
     * @param address The device address of the destination device.
     * @return Return true if the connection is initiated successfully. The connection result
     * is reported asynchronously through the
     * {@code BluetoothGattCallback#onConnectionStateChange(android.bluetooth.BluetoothGatt, int, int)}
     * callback.
     */
    public boolean connect(final String address) {
        if (mBluetoothAdapter == null || address == null) {
            Log.w(tag, "BluetoothAdapter not initialized or unspecified address.");
            return false;
        }

        // Previously connected device.  Try to reconnect.
        if (mBluetoothDeviceAddress != null && address.equals(mBluetoothDeviceAddress)
                && mBluetoothGatt != null) {
            Log.d(tag, "Trying to use an existing mBluetoothGatt for connection.");
            if (mBluetoothGatt.connect()) {
                mConnectionState = STATE_CONNECTING;
                return true;
            } else {
                return false;
            }
        }

        final BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        if (device == null) {
            Log.w(tag, "Device not found.  Unable to connect.");
            return false;
        }
        // We want to directly connect to the device, so we are setting the autoConnect
        // parameter to false.
        mBluetoothGatt = device.connectGatt(this, false, mGattCallback);
        Log.d(tag, "Trying to create a new connection.");
        mBluetoothDeviceAddress = address;
        mConnectionState = STATE_CONNECTING;
        return true;
    }

    /**
     * Disconnects an existing connection or cancel a pending connection. The disconnection result
     * is reported asynchronously through the
     * {@code BluetoothGattCallback#onConnectionStateChange(android.bluetooth.BluetoothGatt, int, int)}
     * callback.
     */
    public void disconnect() {
        if (mBluetoothAdapter == null || mBluetoothGatt == null) {
            Log.w(tag, "BluetoothAdapter not initialized");
            return;
        }
        mBluetoothGatt.disconnect();
    }

    /**
     * After using a given BLE device, the app must call this method to ensure resources are
     * released properly.
     */
    public void close() {
        if (mBluetoothGatt == null) {
            return;
        }
        mBluetoothGatt.close();
        mBluetoothGatt = null;
    }

    /**
     * Request a read on a given {@code BluetoothGattCharacteristic}. The read result is reported
     * asynchronously through the {@code BluetoothGattCallback#onCharacteristicRead(android.bluetooth.BluetoothGatt, android.bluetooth.BluetoothGattCharacteristic, int)}
     * callback.
     *
     * @param characteristic The characteristic to read from.
     */
    public void readCharacteristic(BluetoothGattCharacteristic characteristic) {
        if (mBluetoothAdapter == null || mBluetoothGatt == null) {
            Log.w(tag, "BluetoothAdapter not initialized");
            return;
        }
        mBluetoothGatt.readCharacteristic(characteristic);
    }

    public boolean writeCharacteristic(byte[] byteVal, String uuid, int f) {
        UUID here;
        if (f==0)
            here = UUID_PROJECT_ZERO;
        else
            here = DATA_SERVICE;
        //check mBluetoothGatt is available
        if (mBluetoothGatt == null) {
            Log.e(tag, "lost connection");
            return false;
        }
        BluetoothGattService Service = mBluetoothGatt.getService(here);
        if (Service == null) {
            Log.e(tag, "service not found!");
            return false;
        }
        UUID inputU =
                UUID.fromString(uuid);
        BluetoothGattCharacteristic charac = Service
                .getCharacteristic(inputU);
        //     BluetoothGattCharacteristic charac2 = Service
        //          .getCharacteristic(UUID.fromString(SampleGattAttributes.LED1));
        Log.i(tag, byteVal[0] + "");
        if (charac == null) {
            Log.e(tag, "char not found!");
            return false;
        }


        charac.setValue(byteVal);
        //charac2.setValue(byteVal);
        boolean status = mBluetoothGatt.writeCharacteristic(charac);
        //  mBluetoothGatt.writeCharacteristic(charac2);
        return status;
    }

    public BluetoothGattCharacteristic returnCharacteristic(UUID service, UUID input) {
        BluetoothGattService Service = mBluetoothGatt.getService(service);


        return Service.getCharacteristic(input);
    }

    /**
     * Enables or disables notification on a give characteristic.
     *
     * @param characteristic Characteristic to act on.
     * @param enabled        If true, enable notification.  False otherwise.
     */
    public void setCharacteristicNotification(BluetoothGattCharacteristic characteristic,
                                              boolean enabled) {
        if (mBluetoothAdapter == null || mBluetoothGatt == null) {
            Log.w(TAG, "BluetoothAdapter not initialized");
            return;
        }
        //  Log.i(TAG2, mBluetoothAdapter + "");
        //  Log.i(TAG2, mBluetoothGatt + "");
        //  Log.i(TAG2, characteristic + "");
        //  Log.i(TAG2, enabled + "");
        mBluetoothGatt.setCharacteristicNotification(characteristic, enabled);
        // Log.i(TAG2, mBluetoothAdapter + "");
        if (DATA_SERVICE.equals(characteristic.getUuid())) {
            BluetoothGattDescriptor descriptor = characteristic.getDescriptor(
                    UUID.fromString(SampleGattAttributes.data));
            descriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
            mBluetoothGatt.writeDescriptor(descriptor);
        }
    }


    /**
     * Retrieves a list of supported GATT services on the connected device. This should be
     * invoked only after {@code BluetoothGatt#discoverServices()} completes successfully.
     *
     * @return A {@code List} of supported services.
     */
    public List<BluetoothGattService> getSupportedGattServices() {
        if (mBluetoothGatt == null) return null;

        return mBluetoothGatt.getServices();
    }
    /*
        public String getLED0() {
            UUID service = UUID.fromString("F0001110-0451-4000-B000-000000000000");
            UUID inputU = UUID.fromString("F0001111-0451-4000-B000-000000000000");
            //Log.i(TAG2, "hello");
            if (mNotifyCharacteristic != null) {
                setCharacteristicNotification(
                        mNotifyCharacteristic, false);
                mNotifyCharacteristic = null;
            }
            BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
            setCharacteristicNotification(charac, true);
            readCharacteristic(charac);
            charac.getProperties();
            String getdata = "No data";
            setCharacteristicNotification(charac, false);
            byte[] led0Data = charac.getValue();
            if (led0Data != null && led0Data.length > 0) {
                final StringBuilder sb = new StringBuilder(led0Data.length);
                for (byte LED : led0Data) {
                    sb.append(String.format("%02X ", LED));
                }
                getdata = sb.toString();
            }
            return getdata;
        }
        public String getLED1() {
            UUID service = UUID.fromString("F0001110-0451-4000-B000-000000000000");
            UUID inputU = UUID.fromString("F0001112-0451-4000-B000-000000000000");
          //  Log.i(TAG2, "hello");
            if (mNotifyCharacteristic != null) {
                setCharacteristicNotification(
                        mNotifyCharacteristic, false);
                mNotifyCharacteristic = null;
            }
            BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
            setCharacteristicNotification(charac, true);
            readCharacteristic(charac);
            charac.getProperties();
             setCharacteristicNotification(charac, false);
            String getdata = "No Data";
            byte[] led0Data = charac.getValue();
            if (led0Data != null && led0Data.length > 0) {
                final StringBuilder sb = new StringBuilder(led0Data.length);
                for (byte LED : led0Data) {
                    sb.append(String.format("%02X ", LED));
                }
                getdata = sb.toString();
            }
            return getdata;
        }
    */
    public String getDataService() {
        UUID service = UUID.fromString("F0001130-0451-4000-B000-000000000000");
        UUID inputU = UUID.fromString("F0001131-0451-4000-B000-000000000000");
        //    Log.i(TAG2, "hello");
        if (mNotifyCharacteristic != null) {
            setCharacteristicNotification(
                    mNotifyCharacteristic, false);
            mNotifyCharacteristic = null;
        }
        BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
        setCharacteristicNotification(charac, true);
        readCharacteristic(charac);
        charac.getProperties();
        setCharacteristicNotification(charac, false);
        String getData = "No data";
        byte[] led0Data = charac.getValue();
        if (led0Data == null || led0Data.length<=0)
            return "No Data";
        int value = 0;
        for(int i = 0; i < led0Data.length; i++) {
            value |= led0Data[i] << (led0Data.length - 1 - i) * 8;
        }
     //   value = value + 128;
        String x = String.format("%d", value);
//
//        if (led0Data != null && led0Data.length > 0) {
//            final StringBuilder sb = new StringBuilder(led0Data.length);
//            for (byte LED : led0Data) {
//                sb.append((char)LED);
//                Log.i(TAG2, (char)LED + " " + LED);// + led0Data[1]);
//            }
//             getData = sb.toString();
//           //  getData = getData.replaceAll("\\s+","");
//        }
        // Log.i("life2", getData);
        // if (getData==null || getData.equals("No data") || check==false)
        //0  return getData;
        return x;

        // else
        //     return hexToString(getData);
    }
    public String getSomeDataService() {
        UUID service = UUID.fromString("F0001130-0451-4000-B000-000000000000");
        UUID inputU = UUID.fromString("F0001132-0451-4000-B000-000000000000");
        //    Log.i(TAG2, "hello");
        if (mNotifyCharacteristic != null) {
            setCharacteristicNotification(
                    mNotifyCharacteristic, false);
            mNotifyCharacteristic = null;
        }
        BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
        setCharacteristicNotification(charac, true);
        readCharacteristic(charac);
        charac.getProperties();
        setCharacteristicNotification(charac, false);

        String getData = "No data";
        byte[] led0Data = charac.getValue();

        // if (led0Data != null && led0Data.length > 0) {
        if (led0Data == null || led0Data.length<=0)
            return "No Data";
        final StringBuilder sb = new StringBuilder(led0Data.length);
        float f = ByteBuffer.wrap(led0Data).order(ByteOrder.LITTLE_ENDIAN).getFloat();
        for (byte LED: led0Data){
            sb.append(LED);
        }
        //    for (byte LED : led0Data) {
        //   sb.append((char)LED);
        //   Log.i(TAG2, sb.toString());// +
        Log.i("double" , ByteBuffer.wrap(led0Data).order(ByteOrder.LITTLE_ENDIAN).getFloat() + "The bytes are" + sb.toString());
        //   }
        //    getData = sb.toString();
        //  getData = getData.replaceAll("\\s+","");

        // Log.i("life2", getData);
        // if (getData==null || getData.equals("No data") || check==false)
        return Float.toString(f);

        // else
        //     return hexToString(getData);
    }
    public String getDataStream() {
        UUID service = UUID.fromString("F0001130-0451-4000-B000-000000000000");
        UUID inputU = UUID.fromString("F0001133-0451-4000-B000-000000000000");
        //    Log.i(TAG2, "hello");
        if (mNotifyCharacteristic != null) {
            setCharacteristicNotification(
                    mNotifyCharacteristic, false);
            mNotifyCharacteristic = null;
        }
        BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
        setCharacteristicNotification(charac, true);
        readCharacteristic(charac);
        charac.getProperties();
        setCharacteristicNotification(charac, false);
        String getData = "No data";
        byte[] led0Data = charac.getValue();
        Log.i("DATA", "in here");
        Log.i("DATA", led0Data + " egerrherher[hler[gkerp[gk");
        if (led0Data == null || led0Data.length<=0)
            return "No Data";
        final StringBuilder sb = new StringBuilder(led0Data.length);
        float f = ByteBuffer.wrap(led0Data).order(ByteOrder.LITTLE_ENDIAN).getFloat();

        // Log.i("life2", getData);
        // if (getData==null || getData.equals("No data") || check==false)
        return Float.toString(f);
        // else
        //     return hexToString(getData);
    }
    public String getMoreDataStream() {
        UUID service = UUID.fromString("F0001130-0451-4000-B000-000000000000");
        UUID inputU = UUID.fromString("F0001134-0451-4000-B000-000000000000");
        //    Log.i(TAG2, "hello");
        if (mNotifyCharacteristic != null) {
            setCharacteristicNotification(
                    mNotifyCharacteristic, false);
            mNotifyCharacteristic = null;
        }
        BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
       // setCharacteristicNotification(charac, true);
        readCharacteristic(charac);
        charac.getProperties();
        //setCharacteristicNotification(charac, false);
        String getData = "No data";
        byte[] led0Data = charac.getValue();
        Log.i("DATA", "in here");
        Log.i("DATA", led0Data + " egerrherher[hler[gkerp[gk");
        if (led0Data != null && led0Data.length > 0) {
            final StringBuilder sb = new StringBuilder(led0Data.length);
            for (byte LED : led0Data) {
                sb.append((char)LED);
                Log.i("Notworking", sb.toString() + "  " + LED + "   "  + (char)LED);// + led0Data[1]);
            }
            getData = sb.toString();
            //  getData = getData.replaceAll("\\s+","");
        }
        // Log.i("life2", getData);
        // if (getData==null || getData.equals("No data") || check==false)
        return getData;
        // else
        //     return hexToString(getData);
    }
    public String getDataStreamX() {
        UUID service = UUID.fromString("F0001130-0451-4000-B000-000000000000");
        UUID inputU = UUID.fromString("F0001135-0451-4000-B000-000000000000");
        //    Log.i(TAG2, "hello");
        if (mNotifyCharacteristic != null) {
            setCharacteristicNotification(
                    mNotifyCharacteristic, false);
            mNotifyCharacteristic = null;
        }
        BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
        setCharacteristicNotification(charac, true);
        readCharacteristic(charac);
        charac.getProperties();
        setCharacteristicNotification(charac, false);
        String getData = "No data";
        byte[] led0Data = charac.getValue();
        Log.i("DATA", "in here");
        Log.i("DATA", led0Data + " egerrherher[hler[gkerp[gk");
        if (led0Data == null || led0Data.length<=0)
            return "No Data";
        final StringBuilder sb = new StringBuilder(led0Data.length);
        float f = ByteBuffer.wrap(led0Data).order(ByteOrder.LITTLE_ENDIAN).getFloat();

        // Log.i("life2", getData);
        // if (getData==null || getData.equals("No data") || check==false)
        return Float.toString(f);
        // else
        //     return hexToString(getData);
    }
    public String getDataStreamY() {
        UUID service = UUID.fromString("F0001130-0451-4000-B000-000000000000");
        UUID inputU = UUID.fromString("F0001136-0451-4000-B000-000000000000");
        //    Log.i(TAG2, "hello");
        if (mNotifyCharacteristic != null) {
            setCharacteristicNotification(
                    mNotifyCharacteristic, false);
            mNotifyCharacteristic = null;
        }
        BluetoothGattCharacteristic charac = returnCharacteristic(service, inputU);
        setCharacteristicNotification(charac, true);
        readCharacteristic(charac);
        charac.getProperties();
        setCharacteristicNotification(charac, false);
        String getData = "No data";
        byte[] led0Data = charac.getValue();
        Log.i("DATA", "in here");
        Log.i("DATA", led0Data + " egerrherher[hler[gkerp[gk");
        if (led0Data == null || led0Data.length<=0)
            return "No Data";
        final StringBuilder sb = new StringBuilder(led0Data.length);
        float f = ByteBuffer.wrap(led0Data).order(ByteOrder.LITTLE_ENDIAN).getFloat();

        // Log.i("life2", getData);
        // if (getData==null || getData.equals("No data") || check==false)
        return Float.toString(f);
        // else
        //     return hexToString(getData);
    }


    //TODO: Check write characteristic
    public void readData(){
        running = true;
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(BluetoothLeService.this);
        prefs.edit().putBoolean(preferenceValues.CHECKDATA.name(), false).apply();

        new Thread(new Runnable() {
            public void run() {

                while (running) {

                    try {
                        if (check) {
                            Thread.sleep(1000);
                            check = false;
                        }
                        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(BluetoothLeService.this);
                        String hh =  getMoreDataStream();

                        for (int i=0; i<hh.length(); i++)

                            hh =  hh.charAt(0) + "" + hh.charAt(1) + "" + hh.charAt(2)+"";
                        Log.i("statuslocked", "  "+hh);
                        if (hh.equals("yes")  ) {
                            prefs.edit().putBoolean(preferenceValues.SAVESENDMESSAGE.name(), true).apply();


                            Thread.sleep(1000);
                            String h = "checked";
                            byte[] byteval = new byte[h.length()];
                            for (int i=0; i<h.length(); i++)
                                byteval[i] = (byte)h.charAt(i);
                            writeCharacteristic(byteval, "F0001134-0451-4000-B000-000000000000",1 );
                            Thread.sleep(30000);
                           //while(prefs.getBoolean(preferenceValues.DIALOGCHECKTHREAD.name(),true)){
                           //     Log.i("wtfwtf" , "in here" + "  " + prefs.getBoolean(preferenceValues.DIALOGCHECKTHREAD.name(),true) + " " );
                            //    Thread.sleep(50);
                           // }
                            Log.i("wtfwtf" , "coolstuff"  + prefs.getBoolean(preferenceValues.DIALOGCHECKTHREAD.name(),true) + " " );

                        }
                        //  }


                        Thread.sleep(1500);
                    } catch (Exception e) {
                        Log.i(TAG, "ThreadBluetoothLeservice got fucked " + e);
                    }
                }
            }
        }).start();
    }
}